@@ -1,8 +1,4 @@
 # -*- coding: utf-8 -*-
-from __future__ import (absolute_import, division,
-                        print_function, unicode_literals)
-from builtins import *  # noqa: F401, F403
-
 import os
 import sys
 import shutil
@@ -10,7 +6,7 @@ import re
 import subprocess
 
 from textwrap import dedent
-from collections import namedtuple
+from collections import namedtuple, abc
 
 import pymediainfo
 import guessit
@@ -19,7 +15,7 @@ from requests.exceptions import HTTPError
 
 from .config import config
 from .logging import log
-from .mktorrent import make_torrent
+from .torrent import make_torrent
 from . import tvdb
 from . import imdb
 from .imgur import ImgurUploader
@@ -33,9 +29,21 @@ from .scene import is_scene_crc, query_scene_fname
 
 def format_tag(tag):
     tag = unidecode(tag)
+    if '/' in tag:
+        # Multiple actors can be listed as a single actor like this:
+        # "Thierry Kazazian / Max Mittleman"
+        # (e.g. for "Miraculous: Tales of Ladybug & Cat Noir")
+        tag = tag[:tag.index('/')].strip()
     return tag.replace(' ', '.').replace('-', '.').replace('\'', '.').lower()
 
 
+def format_choices(choices):
+    return ", ".join([
+        str(num) + ": " + value
+        for num, value in enumerate(choices)
+    ])
+
+
 class BbSubmission(Submission):
     default_fields = ("form_title", "tags", "cover")
 
@@ -91,20 +99,80 @@ class BbSubmission(Submission):
             elif choice.lower() == 'y':
                 return True
 
+    def data_method(self, source, target):
+        def copy(source, target):
+            if os.path.isfile(source):
+                return shutil.copy(source, target)
+            if os.path.isdir(source):
+                return shutil.copytree(source, target)
+            raise Exception('Source {} is neither '
+                            'file nor directory'.format(source))
+
+        cat_methods_map = {
+            'movie': ['hard', 'sym', 'copy', 'move'],
+            'tv': ['hard', 'sym', 'copy', 'move'],
+            'music': ['copy', 'move'],
+            }
+
+        method_map = {'hard': os.link,
+                      'sym': os.symlink,
+                      'copy': copy,
+                      'move': shutil.move}
+
+        # use cmd line option if specified
+        option_method = self['options'].get('data_method', 'auto')
+        if option_method != 'auto':
+            method = option_method
+        else:
+            pref_method = config.get('Torrent', 'data_method')
+            if pref_method not in method_map:
+                log.warning(
+                    'Preferred method {} not valid. '
+                    'Choices are {}'.format(pref_method,
+                                            list(method_map.keys())))
+            try:
+                category = self._category
+            except AttributeError:
+                log.warning("{} does not have a category attribute",
+                            type(self).__name__)
+                category = 'movie'  # use movie data methods
+
+            cat_methods = cat_methods_map[category]
+            if pref_method in cat_methods:
+                # use user preferred method if in category method list
+                method = pref_method
+            else:
+                # otherwise use first category method
+                method = cat_methods[0]
+
+        log.notice('Copying data using \'{}\' method', method)
+        return method_map[method](source, target)
+
     @finalize
     @form_field('file_input', 'file')
     def _render_torrentfile(self):
         return make_torrent(self['path'])
 
     def _finalize_torrentfile(self):
+        # move data to upload directory
+        up_dir = config.get('Torrent', 'upload_dir')
+        path_dir, path_base = os.path.split(self['path'])
+        if up_dir and not os.path.samefile(up_dir, path_dir):
+            target = os.path.join(up_dir, path_base)
+            print(target, os.path.exists(target))
+            if not os.path.exists(target):
+                self.data_method(self['path'], target)
+            else:
+                log.notice('Data method target already exists, skipping...')
+
         # black hole
-        out_dir = config.get('Torrent', 'black_hole')
-        if out_dir:
+        bh_dir = config.get('Torrent', 'black_hole')
+        if bh_dir:
             fname = os.path.basename(self['torrentfile'])
-            dest = os.path.join(out_dir, fname)
+            dest = os.path.join(bh_dir, fname)
 
             try:
-                assert os.path.exists(out_dir)
+                assert os.path.exists(bh_dir)
                 assert not os.path.isfile(dest)
             except AssertionError as e:
                 log.error(e)
@@ -140,7 +208,7 @@ class VideoSubmission(BbSubmission):
     default_fields = BbSubmission.default_fields
 
     def _render_guess(self):
-        return {k: v for k, v in guessit.guessit(self['path']).items()}
+        return dict(guessit.guessit(self['path']))
 
     def subcategory(self):
         if type(self) == VideoSubmission:
@@ -150,6 +218,15 @@ class VideoSubmission(BbSubmission):
                 return MovieSubmission
         return type(self)
 
+    def _render_title(self):
+        # Use format "<original title> AKA <english title>" where applicable
+        title_original = self['summary']['title']
+        title_english = self['summary']['titles'].get('XWW', None)
+        if title_english is not None and title_original != title_english:
+            return '{} AKA {}'.format(title_original, title_english)
+        else:
+            return title_original
+
     def _render_tv_specifier(self):
         # if title is specified, look if season/episode are set
         if self['title_arg']:
@@ -178,7 +255,9 @@ class VideoSubmission(BbSubmission):
 
         n = self['options']['num_cast']
         tags = list(self['summary']['genres'])
-        tags += [a['name'] for a in self['summary']['cast'][:n]]
+        tags += [a['name']
+                 for a in self['summary']['cast'][:n]
+                 if a['name']]
         return ",".join(format_tag(tag) for tag in tags)
 
     def _render_mediainfo_path(self):
@@ -188,9 +267,11 @@ class VideoSubmission(BbSubmission):
 
         contained_files = []
         for dp, dns, fns in os.walk(self['path']):
-            contained_files += [
-                os.path.join(dp, fn) for fn in fns
-                if os.path.getsize(os.path.join(dp, fn)) > 10 * 2**20]
+            contained_files += [os.path.join(dp, fn) for fn in fns
+                                if (os.path.getsize(os.path.join(dp, fn))
+                                    > 10 * 2**20)]
+        if len(contained_files) == 1:
+            return contained_files[0]
 
         print("\nWhich file would you like to run mediainfo on? Choices are")
         contained_files.sort()
@@ -217,18 +298,28 @@ class VideoSubmission(BbSubmission):
         try:
             path = self['mediainfo_path']
             if os.name == "nt":
-                return subprocess.Popen([r"mediainfo", path], shell=True,
-                                        stdout=subprocess.PIPE
-                                        ).communicate()[0].decode('utf8')
+                mi = subprocess.Popen([r"mediainfo", path], shell=True,
+                                      stdout=subprocess.PIPE
+                                      ).communicate()[0].decode('utf8')
             else:
-                return subprocess.Popen([r"mediainfo", path],
-                                        stdout=subprocess.PIPE
-                                        ).communicate()[0].decode('utf8')
+                mi = subprocess.Popen([r"mediainfo", path],
+                                      stdout=subprocess.PIPE
+                                      ).communicate()[0].decode('utf8')
         except OSError:
             sys.stderr.write(
                 "Error: Media Info not installed, refer to "
                 "http://mediainfo.sourceforge.net/en for installation")
             exit(1)
+        else:
+            # Replace absolute path with file name
+            mi_dir = os.path.dirname(self['mediainfo_path']) + '/'
+            mi = mi.replace(mi_dir, '')
+
+            # bB's mediainfo parser expects "Xbps" instead of "Xb/s"
+            mi = mi.replace('Kb/s', 'Kbps') \
+                   .replace('kb/s', 'Kbps') \
+                   .replace('Mb/s', 'Mbps')
+            return mi
 
     def _render_tracks(self):
         video_tracks = []
@@ -268,23 +359,22 @@ class VideoSubmission(BbSubmission):
         #          TC, SDTV, DVD5, DVD9, HD-DVD
 
         # todo: replace with guess from self['guess']
-        if 'bluray' in self['path'].lower():
-            return 'BluRay'
-            # todo: 3d
-        elif ('web-dl' in self['path'].lower() or
-              'webdl' in self['path'].lower()):
+        regpath = self['path'].lower().strip('-')
+        if 'bluray' in regpath:
+            return 'BluRay'  # todo: 3d
+        elif 'webdl' in regpath:
             return 'WEB-DL'
-        elif 'webrip' in self['path'].lower():
+        elif 'webrip' in regpath:
             return 'WebRip'
-        elif 'hdtv' in self['path'].lower():
+        elif 'hdtv' in regpath:
             return 'HDTV'
         # elif 'dvdscr' in self['path'].lower():
         #    markers['source'] = 'DVDSCR'
         else:
             print("File:", self['path'])
-            print("Choices:", dict(enumerate(sources)))
+            print("Choices:", format_choices(sources))
             while True:
-                choice = input("Please specify source: ")
+                choice = input("Please specify a source by number: ")
                 try:
                     return sources[int(choice)]
                 except (ValueError, IndexError):
@@ -298,45 +388,72 @@ class VideoSubmission(BbSubmission):
             return 'AVI'
         elif general['format'] == 'MPEG-4':
             return 'MP4'
+        elif general['format'] == 'BDAV':
+            return 'm2ts'
         else:
-            raise Exception("Unknown or unsupported container", general.format)
+            raise RuntimeError("Unknown or unsupported container '{}'".format(
+                general.format))
 
     def _render_video_codec(self):
         video_track = self['tracks']['video']
         # norm_bitrate = (float(bit_rate) /
         #     (video_track.width*video_track.height))
-        if video_track['codec'] in ('V_MPEG4/ISO/AVC', 'AVC'):
-            if ('writing_library' in video_track and
-                    'x264' in video_track['writing_library']):
+        if (video_track['codec_id'] in ('V_MPEG4/ISO/AVC', 'AVC', 'avc1') or
+                video_track['format'] == 'AVC'):
+            if 'x264' in video_track.get('writing_library', ''):
                 return 'x264'
             else:
                 return 'H.264'
-        elif video_track['codec'] == 'XVID':
+        elif video_track['codec_id'] in ('V_MPEGH/ISO/HEVC', 'HEVC'):
+            if 'x265' in video_track.get('writing_library', ''):
+                return 'x265'
+            else:
+                return 'H.265'
+        elif video_track['codec_id'] in ('V_MS/VFW/FOURCC / WVC1',):
+            return 'VC-1'
+        elif 'VP9' in video_track['codec_id']:
+            return 'VP9'
+        elif video_track['codec_id'] == 'XVID':
             return 'XVid'
+        elif video_track['format'] == 'MPEG Video':
+            return 'MPEG-2'
         else:
-            raise Exception("Unknown or unsupported video codec",
-                            video_track['codec'],
-                            video_track['writing_library'])
+            msg = "Unknown or unsupported video codec '{}' ({})".format(
+                            video_track.get('codec_id'),
+                            video_track.get('writing_library'))
+            raise RuntimeError(msg)
 
     def _render_audio_codec(self):
-        audio_codecs = ('AC3', 'DTS', 'FLAC', 'AAC', 'MP3')
-
-        audio_tracks = self['tracks']['audio']
-        audio_track = audio_tracks[0]  # main audio track
+        audio_track = self['tracks']['audio'][0]  # main audio track
+        if (audio_track.get('codec_id_hint') == 'MP3' or
+                audio_track['codec_id'] in ('MPA1L3', '55')):
+            return 'MP3'
+        elif audio_track['codec_id'].lower().startswith('mp4a'):
+            return 'AAC'
+        elif 'Dolby Atmos' in audio_track['commercial_name']:
+            return 'Dolby Atmos'
+        elif 'DTS-HD' in audio_track['commercial_name']:
+            if audio_track.get('other_format', '') == 'DTS XLL X':
+                return 'DTS:X'
+            return 'DTS-HD'
 
+        if audio_track['codec_id'].startswith('A_'):
+            audio_track['codec_id'] = audio_track['codec_id'][2:]
+        audio_codecs = ('AC3', 'EAC3', 'DTS', 'FLAC', 'AAC', 'MP3', 'TRUEHD',
+                        'PCM')
         for c in audio_codecs:
-            if audio_track['codec'].startswith(c):
-                c = c.replace('AC3', 'AC-3')
+            if audio_track['codec_id'].startswith(c):
+                c = c.replace('EAC3', 'AC-3') \
+                     .replace('AC3', 'AC-3') \
+                     .replace('TRUEHD', 'True-HD')
                 return c
 
-        if audio_track['codec'] == 'MPA1L3':
-            return 'MP3'
-
-        raise Exception("Unknown or unsupported audio codec",
-                        audio_track['codec'])
+        raise ValueError("Unknown or unsupported audio codec '{}'".format(
+            audio_track['codec_id']))
 
     def _render_resolution(self):
-        resolutions = ('1080p', '720p', '1080i', '720i', '480p', '480i', 'SD')
+        resolutions = ('2160p', '1080p', '720p', '1080i', '720i',
+                       '480p', '480i', 'SD')
 
         # todo: replace with regex?
         # todo: compare result with mediainfo
@@ -346,9 +463,9 @@ class VideoSubmission(BbSubmission):
                 return res
         else:
             print("File:", self['path'])
-            print("Choices:", dict(enumerate(resolutions)))
+            print("Choices:", format_choices(resolutions))
             while True:
-                choice = input("Please specify resolution: ")
+                choice = input("Please specify a resolution by number: ")
                 try:
                     return resolutions[int(choice)]
                 except (ValueError, IndexError):
@@ -357,29 +474,38 @@ class VideoSubmission(BbSubmission):
 
     def _render_additional(self):
         additional = []
+        video_track = self['tracks']['video']
         audio_tracks = self['tracks']['audio']
         text_tracks = self['tracks']['text']
 
-        for track in audio_tracks[1:]:
-            if 'title' in track and 'commentary' in track['title'].lower():
-                additional.append('w. Commentary')
-
-        if text_tracks:
-            additional.append('w. Subtitles')
         # print [(track.title, track.language) for track in text_tracks]
-
         # todo: rule checking, e.g.
         # main_audio = audio_tracks[0]
         # if (main_audio.language and main_audio.language != 'en' and
         #         not self['tracks']['text']):
         #     raise BrokenRule("Missing subtitles")
 
-        edition = self['guess'].get('edition')
-        if edition:
-            additional.insert(0, edition)
+        if 'remux' in os.path.basename(self['path']).lower():
+            additional.append('REMUX')
 
         if self['guess'].get('proper_count') and self['scene']:
-            additional.insert(0, 'PROPER')
+            additional.append('PROPER')
+
+        edition = self['guess'].get('edition')
+        if isinstance(edition, str):
+            additional.append(edition)
+        elif isinstance(edition, abc.Sequence):
+            additional.extend(edition)
+
+        if 'BT.2020' in video_track.get('color_primaries', ''):
+            additional.append('HDR10')
+
+        for track in audio_tracks[1:]:
+            if 'title' in track and 'commentary' in track['title'].lower():
+                additional.append('w. Commentary')
+                break
+        if text_tracks:
+            additional.append('w. Subtitles')
 
         return additional
 
@@ -403,12 +529,13 @@ class TvSubmission(VideoSubmission):
         'form_description': ('desc', 'text'),
         }
 
+    def _render_guess(self):
+        return dict(guessit.guessit(self['path'],
+                                    options=('--type', 'episode')))
+
     def _render_search_title(self):
         return self['tv_specifier'].title
 
-    def _render_title(self):
-        return self['summary']['title']
-
     @form_field('title')
     def _render_form_title(self):
         markers_list = [self['source'], self['video_codec'],
@@ -465,15 +592,15 @@ class TvSubmission(VideoSubmission):
             [b]Episode title[/b]: {title} ({links})
             [b]Aired[/b]: {air_date} on {network}
             [b]IMDb Rating[/b]: {rating}
-            [b]Director[/b]: {director}
+            [b]Directors[/b]: {directors}
             [b]Writer(s)[/b]: {writers}
             [b]Content rating[/b]: {contentrating}""").format(
                 title=s['episode_title'],
-                links=", ".join(bb.link(*l) for l in links),
+                links=", ".join(bb.link(*l) for l in links),  # noqa: E741
                 air_date=s['air_date'],
                 network=s['network'],
                 rating=rating_bb,
-                director=s['director'],
+                directors=' | '.join(s['directors']),
                 writers=' | '.join(s['writers']),
                 contentrating=s['contentrating']
             )
@@ -489,9 +616,13 @@ class TvSubmission(VideoSubmission):
                 if not e['imdb_id']:
                     return bb.link(e['title'], e['url']) + "\n"
 
-                rating, votes = i.get_rating(e['imdb_id'])
-                return (bb.link(e['title'], e['url']) + "\n" +
-                        bb.s1(bb.format_rating(*rating)))
+                try:
+                    rating, votes = i.get_rating(e['imdb_id'])
+                except ValueError:
+                    return ''
+                else:
+                    return (bb.link(e['title'], e['url']) + "\n" +
+                            bb.s1(bb.format_rating(*rating)))
 
             description += "[b]Episodes[/b]:\n" + bb.list(
                 map(episode_fmt, s['episodes']), style=1)
@@ -530,15 +661,16 @@ class MovieSubmission(VideoSubmission):
         'screenshots': (lambda i, v: 'screenshot' + str(i + 1), 'text'),
         }
 
+    def _render_guess(self):
+        return dict(guessit.guessit(self['path'],
+                                    options=('--type', 'movie')))
+
     def _render_search_title(self):
         if self['title_arg']:
             return self['title_arg']
 
         return self['guess']['title']
 
-    def _render_title(self):
-        return self['summary']['title']
-
     @form_field('title')
     def _render_form_title(self):
         return self['title']
@@ -583,7 +715,7 @@ class MovieSubmission(VideoSubmission):
         [b]Director(s)[/b]: {directors}
         [b]Writer(s)[/b]: {writers}
         [b]Cast[/b]: {cast}""").format(
-            links=", ".join(bb.link(*l) for l in links),
+            links=", ".join(bb.link(*l) for l in links),  # noqa: E741
             name=summary['name'],
             mpaa=summary['mpaa'],
             rating=bb.format_rating(summary['rating'][0],
