@@ -1,9 +1,5 @@
 # -*- coding: utf-8 -*-
-from __future__ import (absolute_import, division,
-                        print_function, unicode_literals)
-from builtins import *  # noqa: F401, F403
 from io import open
-
 from os import path, chmod, makedirs
 
 import configparser
