@@ -1,8 +1,4 @@
 # -*- coding: utf-8 -*-
-from __future__ import (absolute_import, division,
-                        print_function, unicode_literals)
-from builtins import *  # noqa: F401, F403
-
 from attrdict import AttrDict
 import imdbpie
 
@@ -46,11 +42,12 @@ class ImdbResult(object):
     def url(self):
         movie_id = get(self.movie, 'base', 'id')
         if movie_id:
-            return "http://www.imdb.com" + movie_id
+            return "https://www.imdb.com" + movie_id
 
     def summary(self):
         return {
             'title': get(self.movie, 'base', 'title'),
+            'titles': get(self.movie, 'titles'),
             'directors': get(self.movie, 'credits', 'director', default=[]),
             'runtime': self.runtime,
             'rating': (get(self.movie, 'ratings', 'rating'), 10),
@@ -72,7 +69,10 @@ class IMDB(object):
         self.movie = None
 
     def get_rating(self, imdb_id):
-        res = self.imdb.get_title_ratings(imdb_id)
+        try:
+            res = self.imdb.get_title_ratings(imdb_id)
+        except LookupError:
+            res = {}
         return (res.get('rating'), 10), res.get('ratingCount', 0)
 
     def search(self, title):
@@ -85,7 +85,7 @@ class IMDB(object):
 
         while True:
             choice = input('Select number or enter an alternate'
-                           ' search term: [0-%s, 0 default] ' %
+                           ' search term (or an IMDb id): [0-%s, 0 default] ' %
                            (len(results) - 1))
             try:
                 choice = int(choice)
@@ -99,9 +99,15 @@ class IMDB(object):
             except IndexError:
                 pass
             else:
-                imdb_id = result['imdb_id']
-                log.debug("Found IMDb item {}", imdb_id)
-                movie = AttrDict(self.imdb.get_title(imdb_id))
-                movie.credits = self.imdb.get_title_credits(imdb_id)['credits']
-                movie.genres = self.imdb.get_title_genres(imdb_id)['genres']
-                return ImdbResult(movie)
+                log.debug("Found IMDb item {}", result['imdb_id'])
+                return self.get_info(result['imdb_id'])
+
+    def get_info(self, imdb_id):
+        movie = AttrDict(self.imdb.get_title(imdb_id))
+        movie.credits = self.imdb.get_title_credits(imdb_id)['credits']
+        movie.genres = self.imdb.get_title_genres(imdb_id)['genres']
+        title_versions = self.imdb.get_title_versions(imdb_id)
+        movie.titles = {item["region"]: item["title"]
+                        for item in title_versions['alternateTitles']
+                        if "region" in item and "title" in item}
+        return ImdbResult(movie)
